@@ -2,7 +2,7 @@
 
 _license = """
 big
-Copyright 2022 Larry Hastings
+Copyright 2022-2023 Larry Hastings
 All rights reserved.
 
 Permission is hereby granted, free of charge, to any person obtaining a
@@ -29,6 +29,11 @@ import os
 import pathlib
 import re
 
+try:
+    from re import Pattern as re_Pattern
+except ImportError: # pragma: no cover
+    re_Pattern = re._pattern_type
+
 
 __all__ = []
 
@@ -136,12 +141,15 @@ def grep(path, pattern, *, encoding=None, enumerate=False, flags=0):
     For simplicity of implementation, the entire file is read in to memory
     at one time.
 
-    (Tip: to perform a case-insensitive pattern match, pass in the
+    Tip: to perform a case-insensitive pattern match, pass in the
     re.IGNORECASE flag into flags for this function (if pattern is a string
     or bytes) or when creating your regular expression object (if pattern is
-    an re.Pattern object).
+    an re.Pattern object.
+
+    (In older versions of Python, re.Pattern was a private type called
+    re._pattern_type.)
     """
-    if not isinstance(pattern, re.Pattern):
+    if not isinstance(pattern, re_Pattern):
         pattern = re.compile(pattern, flags)
     if isinstance(pattern.pattern, bytes):
         if encoding is not None:
@@ -341,7 +349,7 @@ def translate_filename_to_exfat(s):
         raise ValueError("filename can't be empty")
     s = s.translate(_exfat_translation_table)
     s = s.replace(": ", " - ")
-    s = s.replace(":", "-")
+    s = s.replace(":", ".")
     return s
 
 
