@@ -2,7 +2,7 @@
 
 _license = """
 big
-Copyright 2022 Larry Hastings
+Copyright 2022-2023 Larry Hastings
 All rights reserved.
 
 Permission is hereby granted, free of charge, to any person obtaining a
@@ -34,6 +34,11 @@ import re
 import struct
 import sys
 
+try:
+    from re import Pattern as re_Pattern
+except ImportError: # pragma: no cover
+    re_Pattern = re._pattern_type
+
 
 __all__ = []
 
@@ -81,10 +86,13 @@ def re_partition(s, pattern, count=1, *, flags=0, reverse=False):
 
     If reverse is true, partitions starting at the right,
     like re_rpartition.
+
+    (In older versions of Python, re.Pattern was a private type called
+    re._pattern_type.)
     """
     if reverse:
         return re_rpartition(s, pattern, count, flags=flags)
-    if not isinstance(pattern, re.Pattern):
+    if not isinstance(pattern, re_Pattern):
         pattern = re.compile(pattern, flags=flags)
     if not (isinstance(s, (str, bytes)) and
         (type(s) == type(pattern.pattern))):
@@ -157,8 +165,11 @@ def re_rpartition(s, pattern, count=1, *, flags=0):
 
     If pattern is a string, flags is passed in
     as the flags argument to re.compile.
+
+    (In older versions of Python, re.Pattern was a private type called
+    re._pattern_type.)
     """
-    if not isinstance(pattern, re.Pattern):
+    if not isinstance(pattern, re_Pattern):
         pattern = re.compile(pattern, flags=flags)
     if not (isinstance(s, (str, bytes)) and
         (type(s) == type(pattern.pattern))):
@@ -188,7 +199,7 @@ def re_rpartition(s, pattern, count=1, *, flags=0):
     return tuple(result)
 
 
-# a list of all unicode whitespace characters
+# a list of all unicode whitespace characters known to Python
 _export_name('whitespace')
 whitespace = (
     '\t'    , #     9 0x0009 - tab
@@ -243,8 +254,9 @@ newlines = (
 
     '\r\n'  , # the classic DOS newline sequence
 
-    # '\n\r' sorry, Acorn and RISC OS, you have to add this yourselves.
-    # I'm worried this would cause bugs with a malformed DOS newline.
+    # what about '\n\r'?
+    # sorry, Acorn and RISC OS users, you'll have to add this yourselves.
+    # I'm worried it would cause bugs with a malformed DOS newline.
     )
 
 # this omits the DOS convention '\r\n'
@@ -270,7 +282,7 @@ def _cheap_encode_iterable_of_strings(iterable, encoding='ascii'):
 _export_name('ascii_whitespace')
 ascii_whitespace = _cheap_encode_iterable_of_strings(whitespace, "ascii")
 _export_name('ascii_whitespace_without_dos')
-ascii_whitespace_without_dos = [s for s in ascii_whitespace if s != b'\r\n']
+ascii_whitespace_without_dos = tuple(s for s in ascii_whitespace if s != b'\r\n')
 _export_name('ascii_newlines')
 ascii_newlines   = _cheap_encode_iterable_of_strings(newlines,   "ascii")
 _export_name('ascii_newlines_without_dos')
@@ -280,12 +292,63 @@ ascii_newlines_without_dos = tuple(s for s in ascii_newlines if s != b'\r\n')
 _export_name('utf8_whitespace')
 utf8_whitespace = _cheap_encode_iterable_of_strings(whitespace, "utf-8")
 _export_name('utf8_whitespace_without_dos')
-utf8_whitespace_without_dos = [s for s in utf8_whitespace if s != b'\r\n']
+utf8_whitespace_without_dos = tuple(s for s in utf8_whitespace if s != b'\r\n')
 _export_name('utf8_newlines')
 utf8_newlines   = _cheap_encode_iterable_of_strings(newlines,   "utf-8")
 _export_name('utf8_newlines_without_dos')
 utf8_newlines_without_dos =  tuple(s for s in utf8_newlines if s != b'\r\n')
 
+# reverse an iterable thing.
+# o must be str, bytes, list, tuple, set, or frozenset.
+# value returned is the same type as o.
+#
+# we don't need to bother checking the type of o.
+# _multisplit_reversed is an internal function
+# and I've manually checked every call site.
+def _multisplit_reversed(o, name='s'):
+    if isinstance(o, str):
+        if len(o) <= 1:
+            return o
+        return "".join(reversed(o))
+    if isinstance(o, bytes):
+        if len(o) <= 1:
+            return o
+        return b"".join(o[i:i+1] for i in range(len(o)-1, -1, -1))
+    # assert isinstance(o, (list, tuple, set, frozenset))
+    t = type(o)
+    return t(_multisplit_reversed(p) for p in o)
+
+
+# _reversed_builtin_separators precalculates the reversed versions
+# of the builtin separators.  we use the reversed versions when
+# reverse=True.  this is a minor speed optimization, but also it
+# helps a lot with the lrucache for _separators_to_re.
+#
+# we test that these cached versions are correct in tests/test_text.py.
+#
+_reversed_utf8_whitespace_without_dos = _multisplit_reversed(utf8_whitespace_without_dos)
+_reversed_utf8_newlines_without_dos =   _multisplit_reversed(utf8_newlines_without_dos)
+
+_reversed_builtin_separators = {
+    whitespace: whitespace_without_dos + ('\n\r',),
+    whitespace_without_dos: whitespace_without_dos,
+
+    newlines: newlines_without_dos + ("\n\r",),
+    newlines_without_dos: newlines_without_dos,
+
+    ascii_whitespace: ascii_whitespace_without_dos + (b"\n\r",),
+    ascii_whitespace_without_dos: ascii_whitespace_without_dos,
+
+    ascii_newlines: ascii_newlines_without_dos + (b"\n\r",),
+    ascii_newlines_without_dos: ascii_newlines_without_dos,
+
+    utf8_whitespace: _reversed_utf8_whitespace_without_dos + (b"\n\r",),
+    utf8_whitespace_without_dos: _reversed_utf8_whitespace_without_dos,
+
+    utf8_newlines: _reversed_utf8_newlines_without_dos + (b"\n\r",),
+    utf8_newlines_without_dos: _reversed_utf8_newlines_without_dos,
+    }
+
 
 def _re_quote(s):
     # don't bother escaping whitespace.
@@ -338,7 +401,8 @@ def _separators_to_re(separators, as_bytes, separate=False, keep=False):
         hash(separators)
     except TypeError:
         separators = tuple(separators)
-    return __separators_to_re(separators, as_bytes, separate=separate, keep=keep)
+    return __separators_to_re(separators, bool(as_bytes), separate=bool(separate), keep=bool(keep))
+
 
 
 @_export
@@ -366,20 +430,37 @@ def multistrip(s, separators, left=True, right=True):
     separators stripped.  (If left and right are both
     false, returns s unchanged.)
     """
-    if not (left or right):
-        return s
-    as_bytes = isinstance(s, bytes)
+    type_of_s = type(s)
+    as_bytes = type_of_s == bytes
+
+    if not (as_bytes or (type_of_s == str)):
+        raise TypeError('s must be str or bytes')
+
+    type_of_separators = type(separators)
     if as_bytes:
         head = b'^'
         tail = b'$'
-        if isinstance(separators, bytes):
+        if type_of_separators == bytes:
             # not iterable of bytes, literally a bytes string.
             # split it ourselves.  otherwise, _separators_to_re will
             # iterate over it, and... oops! it gets bytes!
+            check_separators = False
             separators = tuple(separators[i:i+1] for i in range(len(separators)))
+        else:
+            check_separators = True
     else:
         head = '^'
         tail = '$'
+        check_separators = type_of_separators != str
+
+    if check_separators:
+        for o in separators:
+            if not isinstance(o, type_of_s):
+                raise TypeError("separators must be an iterable of objects the same type as s")
+
+    if not (left or right):
+        return s
+
     pattern = _separators_to_re(separators, as_bytes, separate=False, keep=False)
 
     start = 0
@@ -388,11 +469,13 @@ def multistrip(s, separators, left=True, right=True):
         left_match = re.match(head + pattern, s)
         if left_match:
             start = left_match.end(0)
+            s = s[start:]
     if right:
         right_match = re.search(pattern + tail, s)
         if right_match:
             end = right_match.start(0)
-    return s[start:end]
+            s = s[:end]
+    return s
 
 
 # for keep
@@ -402,14 +485,12 @@ ALTERNATING="ALTERNATING"
 _export_name(ALTERNATING)
 
 # for strip
-NOT_SEPARATE = "NOT_SEPARATE"
-_export_name(NOT_SEPARATE)
 LEFT = "LEFT"
 _export_name(LEFT)
 RIGHT = "RIGHT"
 _export_name(RIGHT)
-STR_SPLIT = "STR_SPLIT"  # not implemented yet
-_export_name(STR_SPLIT)
+PROGRESSIVE = "PROGRESSIVE"
+_export_name(PROGRESSIVE)
 
 @_export
 def multisplit(s, separators=None, *,
@@ -417,27 +498,30 @@ def multisplit(s, separators=None, *,
     maxsplit=-1,
     reverse=False,
     separate=False,
-    strip=NOT_SEPARATE,
+    strip=False,
     ):
     """
-    Like str.split, supporting multiple separator strings and tunable behavior.
+    Splits strings like str.split, but with multiple separators and options.
 
     s can be str or bytes.
 
     separators should be an iterable of str or bytes, matching s.
 
     Returns an iterator yielding the strings split from s.  If keep
-    is not false, and strip is false, joining these strings together
-    will recreate s.
+    is true (or ALTERNATING), and strip is false, joining these strings
+    together will recreate s.
 
-    multisplit is greedy; if two (or more) separator strings match at a
-    particular index in "s", it splits using the largest one.
+    multisplit is *greedy*: if two or more separators start at the same
+    location in "s", multisplit splits using the longest matching separator.
+    For example:
+        big.multisplit('wxabcyz', ('a', 'abc'))
+    yields 'wx' then 'yz'.
 
     "keep" indicates whether or not multisplit should keep the separator
     strings.  It supports four values:
         false (the default)
             Discard the separators.
-        true (apart from ALTERNATING or AS_PAIRS)
+        true (apart from ALTERNATING and AS_PAIRS)
             Append the separators to the end of the split strings.
             You can recreate the original string by passing the
             list returned in to ''.join.
@@ -472,67 +556,104 @@ def multisplit(s, separators=None, *,
     "strip" indicates whether multisplit should strip separators from
     the beginning and/or end of s, a la multistrip.  It supports
     six values:
-        NOT_SEPARATE (the default)
-            Use the opposite value specified for "separate"
-            If separate=True, behaves as if strip=False.
-            If separate=False, behaves as if strip=True.
-        false
+        false (the default)
             Don't strip separators from the beginning or end of s.
+        true (apart from LEFT, RIGHT, and PROGRESSIVE)
+            Strip separators from the beginning and end of s
+            (a la str.strip).
         LEFT
             Strip separators only from the beginning of s
             (a la str.lstrip).
         RIGHT
             Strip separators only from the end of s
             (a la str.rstrip).
-        true
-            Strip separators from the beginning and end of s
-            (a la str.strip).
-        STR_SPLIT (currently unimplemented, equivalent to true)
-            Strip the way str.split does when splitting on whitespace.
+        PROGRESSIVE
             Strip from the beginning and end of s, unless maxsplit
             is nonzero and the entire string is not split.  If
             splitting stops due to maxsplit before the entire string
             is split, and reverse is false, don't strip the end of
             the string. If splitting stops due to maxsplit before
             the entire string is split, and reverse is true, don't
-            strip the beginning of the string.
+            strip the beginning of the string.  (This is how str.strip
+            and str.rstrip behave when sep=None.)
 
     "maxsplit" should be either an integer or None.  If maxsplit is an
     integer greater than -1, multisplit will split s no more than
     maxsplit times.
 
-    "reverse" affects whether the "maxsplit" splits start at the
-    beginning or the end of the string.  It supports two values:
+    "reverse" controls whether multisplit splits starting from the
+    beginning or from the end of the string.  It supports two values:
         false (the default)
-            Start splitting at the beginning of the string.
+            Start splitting from the beginning of the string
+            and scanning right.
         true
-            Start splitting at the end of the string.
-    "reverse" has no effect when maxsplit is 0, -1, or None.
+            Start splitting from the end of the string and
+            scanning left.
+    Splitting from the end of the string and scanning left has two
+    effects.  First, if maxsplit is a number greater than 0,
+    the splits will start at the end of the string rather than
+    the beginning.  Second, if there are overlapping instances of
+    separators in the string, multisplit will prefer the rightmost
+    separator rather than the left.  For example:
+        multisplit(" x x ", (" x ",), keep=big.ALTERNATING)
+    will split on the leftmost instance of " x ", yielding
+        "", " x ", "x "
+    whereas
+        multisplit(" x x ", (" x ",), keep=big.ALTERNATING, reverse=True)
+    will split on the rightmost instance of " x ", yielding
+        " x", " x ", ""
     """
+    type_of_s = type(s)
+    as_bytes = type_of_s == bytes
+    if not (as_bytes or (type_of_s == str)):
+        raise TypeError('s must be str or bytes')
 
-    as_bytes = isinstance(s, bytes)
     if separators is None:
         separators = ascii_whitespace if as_bytes else whitespace
+        check_separators = False
+    elif not separators:
+        raise TypeError("separators must be either None or an iterable of objects the same type as s")
+    else:
+        check_separators = True
 
+    type_of_separators = type(separators)
     if as_bytes:
-        if isinstance(separators, bytes):
+        if type_of_separators == bytes:
             # not iterable of bytes, literally a bytes string.
             # split it ourselves.
             separators = tuple(separators[i:i+1] for i in range(len(separators)))
-        separators_type = bytes
+            check_separators = False
         empty = b''
     else:
-        separators_type = str
         empty = ''
-    for o in separators:
-        if not isinstance(o, separators_type):
-            raise ValueError("separators must be an iterable of objects the same type as s")
+        if check_separators:
+            check_separators = type_of_separators != str
 
-    if strip == NOT_SEPARATE:
-        strip = not separate
+    # check_separators is True if separators isn't str or bytes
+    # or something we split ourselves.
+    if check_separators:
+        for o in separators:
+            if not isinstance(o, type_of_s):
+                raise TypeError("separators must be either None or an iterable of objects the same type as s")
+
+    separators_to_re_keep = keep
 
     if strip:
-        s = multistrip(s, separators, left=(strip != RIGHT), right=(strip != LEFT))
+        if strip == PROGRESSIVE:
+            if maxsplit == -1:
+                strip = left = right = True
+            else:
+                left = not reverse
+                right = reverse
+                separators_to_re_keep = True
+        else:
+            left = strip != RIGHT
+            right = strip != LEFT
+        s = multistrip(s, separators, left=left, right=right)
+        if not s:
+            # oops! all separators!
+            # this will make us exit early, just a few lines down from here.
+            maxsplit = 0
 
     if maxsplit == None:
         maxsplit = -1
@@ -548,72 +669,92 @@ def multisplit(s, separators=None, *,
     # convert maxsplit for use with re.split.
     #
     # re.split interprets maxsplit slightly differently:
-    # maxsplit==0 means "allow all splits".
-    # maxsplit==1 means "only allow one split".
+    #   its maxsplit==0 means "allow all splits".
+    #   its maxsplit==1 means "only allow one split".
     #
     # (re.split doesn't have a way to express
     #  "don't split" with its maxsplit parameter,
     #  which is why we handled it already.)
-    if maxsplit == -1:
-        maxsplit = 0
-
-    # we may change the values of keep and maxsplit
-    # we pass in to subfunctions, depending.
-    # these are the original values.
-    original_keep = keep
-    original_maxsplit = maxsplit
-
-    if reverse and (maxsplit > 0):
-        # we have to handle reverse maxsplit by hand.
-        # we completely split, then rejoin strings
-        # together to simulate the maxsplit value.
-        # this means that inside the function we have
-        # to keep separators; if the user didn't specify
-        # keep, we'll throw the un-joined ones away.
-        maxsplit = 0
-        keep = True
-    else:
-        # we're not maxsplitting, so reverse is unused.
-        reverse = False
-
-    pattern = _separators_to_re(separators, as_bytes, keep=keep, separate=separate)
-
-    l = re.split(pattern, s, maxsplit)
-    assert l
+    re_split_maxsplit = maxsplit if maxsplit != -1 else 0
 
     if reverse:
-        maxsplit = original_maxsplit
-        keep = original_keep
-        # alternating nonsep, sep strings
-        desired_length = 1 + (2*maxsplit)
+        # if reverse is true, when separators overlap,
+        # we need to prefer the rightmost one rather than
+        # the leftmost one.  how do we do *that*?
+        # Eric Smith had the brainstorm: reverse the string
+        # and the separators, split, and reverse the output
+        # and the strings in the output.
+        s = _multisplit_reversed(s, 's')
+        separators = tuple(separators)
+        s2 = _reversed_builtin_separators.get(separators, None)
+        if s2 != None:
+            separators = s2
+        else:
+            separators = _multisplit_reversed(separators, 'separators')
+
+    pattern = _separators_to_re(separators, as_bytes, keep=separators_to_re_keep, separate=separate)
+    # print("PATTERN", pattern, f"{separators_to_re_keep=} {separate=}")
+
+    l = re.split(pattern, s, re_split_maxsplit)
+    assert l
+    # print("S", repr(s), "L", l, f"{re_split_maxsplit=}")
+
+    if strip == PROGRESSIVE:
+        # l alternates nonsep and sep strings.
+        # it's always an odd length, starting and ending with nonsep.
         length = len(l)
-        join_count = length - desired_length + 1
-        if join_count > 1:
-            s = empty.join(l[:join_count])
-            del l[:join_count]
-            l.insert(0, s)
+        assert length & 1
+
+        desired_length = 1 + (2*maxsplit)
+
+        # dang! this is complicated!
+        # maxsplit has to extend *past* the last nonsep
+        # for us to strip on the far side.
+        #  ' a b c   '.split(None, maxsplit=2) => ['a', 'b', 'c   ']
+        #  ' a b c   '.split(None, maxsplit=3) => ['a', 'b', 'c']
+        for i in range(length - 1, 0, -2):
+            nonsep = l[i]
+            if nonsep:
+                last_non_empty_nonsep = i
+                break
+        else:
+            last_non_empty_nonsep = 0
+
+        if desired_length > (last_non_empty_nonsep + 2):
+            # strip!
+            l = l[:last_non_empty_nonsep + 1]
+            desired_length = length = last_non_empty_nonsep
+
         if not keep:
             for i in range(len(l) - 2, 0, -2):
                 del l[i]
 
+    if reverse:
+        l = _multisplit_reversed(l, 'l')
+        l.reverse()
+
     if not keep:
-        for o in l:
-            yield o
+        l.reverse()
+        while l:
+            yield l.pop()
         return
 
     # from here on out, we're 'keep'-ing the separator strings.
     # (we're returning the separator strings in one form or another.)
 
     if keep == ALTERNATING:
-        for o in l:
-            yield o
+        l.reverse()
+        while l:
+            yield l.pop()
         return
 
     if (len(l) % 2) == 1:
         l.append(empty)
 
     previous = None
-    for o in l:
+    l.reverse()
+    while l:
+        o = l.pop()
         if previous is None:
             previous = o
             continue
@@ -632,27 +773,34 @@ def multipartition(s, separators, count=1, *, reverse=False, separate=True):
 
     "s" can be str or bytes.
 
-    "separators" should be an iterable of str or bytes, matching "s".
+    "separators" should be an iterable of objects of the same type as "s".
 
     By default, if any of the strings in "separators" are found in "s",
     returns a tuple of three strings: the portion of "s" leading up to
     the earliest separator, the separator, and the portion of "s" after
     that separator.  Example:
 
-    multipartition('aXbYz', ('X', 'Y')) => ('a', 'X', 'bYz')
+        multipartition('aXbYz', ('X', 'Y')) => ('a', 'X', 'bYz')
 
     If none of the separators are found in the string, returns
     a tuple containing `s` unchanged followed by two empty strings.
 
+    multipartition is *greedy*: if two or more separators appear at
+    the leftmost location in `s`, multipartition partitions using
+    the longest matching separator.  For example:
+
+        big.multipartition('wxabcyz', ('a', 'abc')) => ('wx', 'abc', 'yz')
+
     Passing in an explicit "count" lets you control how many times
     multipartition partitions the string.  multipartition will always
     return a tuple containing (2*count)+1 elements.  Passing in a
     count of 0 will always return a tuple containing s.
 
-    If separate is true, multiple adjacent separator strings behave
+    If `separate` is true, multiple adjacent separator strings behave
     like one separator.  Example:
 
-    multipartition('aXYbXYc', ('X', 'Y',), separate=True) => ('a', 'XY', 'bXYc')
+        big.text.multipartition('aXYbYXc', ('X', 'Y',), count=2, separate=False) => ('a', 'XY', 'b', 'YX', 'c')
+        big.text.multipartition('aXYbYXc', ('X', 'Y',), count=2, separate=True ) => ('a', 'X', '', 'Y', 'bYXc')
 
     If reverse is true, multipartition behaves like str.rpartition.
     It partitions starting on the right, scanning backwards through
@@ -681,6 +829,10 @@ def multipartition(s, separators, count=1, *, reverse=False, separate=True):
             result.extend(extension)
     return tuple(result)
 
+@_export
+def multirpartition(s, separators, count=1, *, reverse=False, separate=True):
+    return multipartition(s, separators, count=count, reverse=not reverse, separate=separate)
+
 
 _invalid_state = "_invalid_state"
 _in_word = "_in_word"
@@ -699,7 +851,8 @@ def gently_title(s, *, apostrophes=None, double_quotes=None):
     any blob of non-whitespace characters.)
 
     Capitalize the letter after an apostrophe if
-        a) the apostrophe is after whitespace
+        a) the apostrophe is after whitespace or a
+           left parenthesis character ('(')
            (or is the first letter of the string), or
         b) if the apostrophe is after a letter O or D,
            and that O or D is after whitespace (or is
@@ -749,6 +902,7 @@ def gently_title(s, *, apostrophes=None, double_quotes=None):
         if not double_quotes:
             double_quotes = b'"'
         d_and_o = b'DO'
+        lparen = b'('
         characters_in_s = (s[i:i+1] for i in range(len(s)))
     else:
         empty = ""
@@ -757,6 +911,7 @@ def gently_title(s, *, apostrophes=None, double_quotes=None):
         if not double_quotes:
             double_quotes = '"“”„‟«»‹›'
         d_and_o = 'DO'
+        lparen = '('
         characters_in_s = iter(s)
 
     # ooh, fancy!
@@ -767,7 +922,9 @@ def gently_title(s, *, apostrophes=None, double_quotes=None):
     result = []
     state = _after_whitespace
     for c in characters_in_s:
-        is_space = c.isspace()
+        original_c = c
+        original_state = state
+        is_space = c.isspace() or (c == lparen)
         is_apostrophe = _is_apostrophe(c)
         is_double_quote = _is_double_quote(c)
         if state == _in_word:
@@ -794,6 +951,7 @@ def gently_title(s, *, apostrophes=None, double_quotes=None):
         elif state == _after_whitespace_then_D_or_O_then_apostrophe:
             c = c.upper()
             state = _in_word
+        # print(f"  {original_c!r} {repr(is_space):5} {original_state!r:49} -> {c!r} {state!r}")
         result.append(c)
     return empty.join(result)
 
@@ -1011,7 +1169,8 @@ def split_quoted_strings(s, quotes=None, *, triple_quotes=True, backslash=None):
 @_export
 class LineInfo:
     """
-    The info object yielded by a lines iterator.
+    The second object yielded by a lines iterator,
+    containing metadata about the line.
     You can add your own fields by passing them in
     via **kwargs; you can also add new attributes
     or modify existing attributes as needed from
@@ -1077,11 +1236,11 @@ class lines:
 
         tab_width is not used by lines itself, but is stored internally and
         may be used by other lines modifier functions
-        (e.g. lines_convert_tabs_to_spaces). Similarly, all keyword
-        arguments passed in via kwargs are stored internally and can be
-        accessed by user-defined lines modifier functions.
+        (e.g. lines_convert_tabs_to_spaces, lines_strip_indent). Similarly,
+        all keyword arguments passed in via kwargs are stored internally
+        and can be accessed by user-defined lines modifier functions.
 
-        Composable with all the lines_ functions from the big.text module.
+        Composable with all the lines_ modifier functions in the big.text module.
         """
         if not isinstance(line_number, int):
             raise TypeError("line_number must be int")
@@ -1128,8 +1287,7 @@ def lines_rstrip(li):
     A lines modifier function.  Strips trailing whitespace from the
     lines of a "lines iterator".
 
-    Composable with all the lines_ functions
-    from the big text module.
+    Composable with all the lines_ modifier functions in the big.text module.
     """
     for info, line in li:
         yield (info, line.rstrip())
@@ -1144,7 +1302,7 @@ def lines_strip(li):
     a field to the associated LineInfo object:
       * leading - the leading whitespace string that was removed
 
-    Composable with all the lines_ functions from the big.text module.
+    Composable with all the lines_ modifier functions in the big.text module.
     """
     for info, line in li:
         lstripped = line.lstrip()
@@ -1179,7 +1337,7 @@ def lines_filter_comment_lines(li, comment_separators):
         comments inside quoted strings.  It truncates the
         line but still always yields the line.
 
-    Composable with all the lines_ functions from the big.text module.
+    Composable with all the lines_ modifier functions in the big.text module.
     """
     if not comment_separators:
         raise ValueError("illegal comment_separators")
@@ -1200,7 +1358,7 @@ def lines_filter_comment_lines(li, comment_separators):
 
 
 @_export
-def lines_filter_contains(li, s, *, invert=False):
+def lines_containing(li, s, *, invert=False):
     """
     A lines modifier function.  Only yields lines
     that contain s.  (Filters out lines that
@@ -1209,7 +1367,7 @@ def lines_filter_contains(li, s, *, invert=False):
     If invert is true, returns the opposite--
     filters out lines that contain s.
 
-    Composable with all the lines_ functions from the big.text module.
+    Composable with all the lines_ modifier functions in the big.text module.
     """
     if invert:
         for t in li:
@@ -1221,7 +1379,7 @@ def lines_filter_contains(li, s, *, invert=False):
             yield t
 
 @_export
-def lines_filter_grep(li, pattern, *, invert=False, flags=0):
+def lines_grep(li, pattern, *, invert=False, flags=0):
     """
     A lines modifier function.  Only yields lines
     that match the regular expression pattern.
@@ -1235,8 +1393,11 @@ def lines_filter_grep(li, pattern, *, invert=False, flags=0):
     filters out lines that match pattern.
 
     Composable with all the lines_ functions from the big.text module.
+
+    (In older versions of Python, re.Pattern was a private type called
+    re._pattern_type.)
     """
-    if not isinstance(pattern, re.Pattern):
+    if not isinstance(pattern, re_Pattern):
         pattern = re.compile(pattern, flags=flags)
     search = pattern.search
     if invert:
@@ -1259,7 +1420,7 @@ def lines_sort(li, *, reverse=False):
     If reverse is true, lines are sorted
     from highest to lowest.
 
-    Composable with all the lines_ functions from the big.text module.
+    Composable with all the lines_ modifier functions in the big.text module.
     """
     lines = list(li)
     lines.sort(key=lambda t:t[1], reverse=reverse)
@@ -1303,7 +1464,7 @@ def lines_strip_comments(li, comment_separators, *, quotes=('"', "'"), backslash
         comments inside quoted strings.  It truncates the
         line but still always yields the line.
 
-    Composable with all the lines_ functions from the big.text module.
+    Composable with all the lines_ modifier functions in the big.text module.
     """
     if not comment_separators:
         raise ValueError("illegal comment_separators")
@@ -1359,7 +1520,7 @@ def lines_convert_tabs_to_spaces(li):
     A lines modifier function.  Converts tabs to spaces for the lines
     of a "lines iterator", using the tab_width passed in to lines.
 
-    Composable with all the lines_ functions from the big.text module.
+    Composable with all the lines_ modifier functions in the big.text module.
     """
     for info, line in li:
         yield (info, line.expandtabs(li.tab_width))
@@ -1373,6 +1534,7 @@ def lines_strip_indent(li):
     Sets two new fields on the associated LineInfo object for every line:
       * indent - an integer indicating how many indents it's observed
       * leading - the leading whitespace string that was removed
+    Also updates LineInfo.column_number as needed.
 
     Uses an intentionally simple algorithm.
     Only understands tab and space characters as indent characters.
@@ -1447,7 +1609,7 @@ def lines_filter_empty_lines(li):
         (3, "a")
         (5, "b")
 
-    Composable with all the lines_ functions from the big.text module.
+    Composable with all the lines_ modifier functions in the big.text module.
     """
     for t in li:
         if not t[1]:
