@@ -2,7 +2,7 @@
 
 _license = """
 big
-Copyright 2022 Larry Hastings
+Copyright 2022-2023 Larry Hastings
 All rights reserved.
 
 Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,7 +24,7 @@ OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 """
 
-import big
+import big.all as big
 import os.path
 import pathlib
 import re
@@ -184,7 +184,7 @@ class BigFileTmpdirTests(unittest.TestCase):
     def test_translate_filename_to_exfat(self):
         self.assertEqual(big.translate_filename_to_exfat("abcde"), "abcde")
         before = 'abc\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f/\\:*?": <>|xyz'
-        after = "abc@?01?02?03?04?05?06?07?08?09?0a?0b?0c?0d?0e?0f?10?11?12?13?14?15?16?17?18?19?1a?1b?1c?1d?1e?1f---@.' - {}!xyz"
+        after = "abc@?01?02?03?04?05?06?07?08?09?0a?0b?0c?0d?0e?0f?10?11?12?13?14?15?16?17?18?19?1a?1b?1c?1d?1e?1f--.@.' - {}!xyz"
         self.assertEqual(big.translate_filename_to_exfat(before), after)
 
         with self.assertRaises(ValueError):
